const { refObject, Button, ImageButton, TextBox, Vector, world } = require('@tabletop-playground/api');
const { SetIdObject } = require('./general/General_Functions.js');

refObject.onCreated.add(() => {
  SetIdObject(refObject.getId());
})
//-----------------------------------------------------------------
const zPosition = 0.1;
const widgetWidth = 1600;
const widgetHeight = 1000;

let countSkills = 19;
let majorSkills = [];
let baffSkills = [];
let debaffSkills = [];
let mainSkills = [];
let startValueMainSkills = [];
for (let i = 0; i < countSkills; i++) {
  majorSkills["major" + i] = new TextBox().setText("0").setEnabled(false);
  majorSkills["major" + i].setFontSize(40);
  majorSkills["major" + i].onTextChanged.add(() => {
    saveState();
  })
  //--
  baffSkills["baff" + i] = new TextBox().setText("0").setInputType(4);
  baffSkills["baff" + i].setFontSize(40);
  baffSkills["baff" + i].onTextChanged.add(() => {
    RecalculationMajorSkills(i);
  })
  //--
  debaffSkills["debaff" + i] = new TextBox().setText("0").setInputType(4);
  debaffSkills["debaff" + i].setFontSize(40);
  debaffSkills["debaff" + i].onTextChanged.add(() => {
    RecalculationMajorSkills(i);
  })
  //--
  mainSkills["main" + i] = new TextBox().setText("0").setInputType(4);
  mainSkills["main" + i].setFontSize(40);
  mainSkills["main" + i].onTextChanged.add(() => {
    RecalculationMajorSkills(i);
    //characteristics.RecalculationFreePoints();
  })
  //--
  startValueMainSkills[i] = 0;
}
//-----------------
function RecalculationMajorSkills(index) {
  let main = startValueMainSkills[index] + parseInt(mainSkills["main" + index].getText());
  let debaff = parseInt(debaffSkills["debaff" + index].getText());
  let baff = parseInt(baffSkills["baff" + index].getText());
  majorSkills["major" + index].setText(main + baff - debaff);
}

function CreateChangerTextBox(parent, index, position, array, isMain) {
  let imageSize = 100;
  let buttonOffsetY = 10;

  let decrement = new ImageButton().setImage("minus.png");
  decrement.setImageSize(imageSize);
  decrement.onClicked.add(() => {
    let currentValue = array[index].getText();
    if (currentValue > 0) {
      array[index].setText(currentValue - 1);
    }
  })
  parent.addChild(decrement, position.x - 60, position.y + buttonOffsetY, 50, 50);
  //--
  parent.addChild(array[index], position.x, position.y, 80, 65);
  //--
  let increment = new ImageButton().setImage("plus.png");
  increment.setImageSize(imageSize);
  increment.onClicked.add(() => {
    let currentValue = parseInt(array[index].getText());
    array[index].setText(currentValue + 1);
  })
  parent.addChild(increment, position.x + 90, position.y + buttonOffsetY, 50, 50);
}
//-----------------------------------------------------------------
class SkillsMajor {
  constructor(parent, position) {
    this.parent = parent;
    this.showPosition = position;
    this.hidePosition = position.add(new Vector(0, 0, -zPosition));
    //-------------------------
    this.nC = new Canvas();

    this.nCUI = new UIElement();
    this.nCUI.useWidgetSize = false;
    this.nCUI.position = position;
    this.nCUI.rotation = new Rotator(0, 0, 180);
    this.nCUI.widget = this.nC;
    this.nCUI.width = widgetWidth;
    this.nCUI.height = widgetHeight;
    this.nCUI.scale = 0.1;
    parent.attachUI(this.nCUI);
    //-------------------------
    let offsetY = 0; let offsetX = 440;
    for (let i = 0; i < countSkills; i++, offsetY++) {
      this.nC.addChild(majorSkills["major" + i], offsetX, 150 + offsetY * 70, 80, 70);
      CreateChangerTextBox(this.nC, "main" + i, position.add(new Vector(offsetX + 180, 150 + offsetY * 70, 0)), mainSkills, true);
      if (i == 9) { offsetY = -1; offsetX = 1170; }
    }
    //-------------------------
    let borderMain = new Border();
    borderMain.setColor(new Color(0.05, 1, 0.05));
    this.nC.addChild(borderMain, 640, 100, 40, 40);
    this.nC.addChild(borderMain, 1370, 100, 40, 40);
    //-------------------------
    let arrow = new ImageButton().setImage("strelka.png");
    arrow.setImageSize(50);
    this.nC.addChild(arrow, 1525, 25, 50, 50);
    //-------------------------
    let t = this;
    arrow.onClicked.add(function () {
      t.HideUI();
      skillsChanger.ShowUI();
    });
  }

  HideUI() {
    this.nCUI.position = this.hidePosition;
    this.parent.updateUI(this.nCUI);
  }
  ShowUI() {
    this.nCUI.position = this.showPosition;
    this.parent.updateUI(this.nCUI);
  }
}
let skillsMajor = new SkillsMajor(refObject, new Vector(0, 0, zPosition));

class SkillsChanger {
  constructor(parent, position) {
    this.parent = parent;
    this.showPosition = position.add(new Vector(0, 0, zPosition));
    this.hidePosition = position;
    //-------------------------
    this.nC = new Canvas();

    this.nCUI = new UIElement();
    this.nCUI.useWidgetSize = false;
    this.nCUI.position = position;
    this.nCUI.rotation = new Rotator(0, 0, 180);
    this.nCUI.widget = this.nC;
    this.nCUI.width = widgetWidth;
    this.nCUI.height = widgetHeight;
    this.nCUI.scale = 0.1;
    parent.attachUI(this.nCUI);
    //-------------------------
    let offsetY = 0; let offsetX = 500;
    for (let i = 0; i < countSkills; i++, offsetY++) {
      CreateChangerTextBox(this.nC, "baff" + i, position.add(new Vector(offsetX, 150 + offsetY * 70, 0)), baffSkills);
      CreateChangerTextBox(this.nC, "debaff" + i, position.add(new Vector(offsetX + 210, 150 + offsetY * 70, 0)), debaffSkills);
      if (i == 9) { offsetY = -1; offsetX = 1230; }
    }
    //-------------------------
    let borderBaff = new Border();
    borderBaff.setColor(new Color(0.05, 0, 1));
    this.nC.addChild(borderBaff, 520, 100, 40, 40);
    this.nC.addChild(borderBaff, 1250, 100, 40, 40);
    //-------------------------
    let borderDebaff = new Border();
    borderDebaff.setColor(new Color(1, 0, 0.05));
    this.nC.addChild(borderDebaff, 730, 100, 40, 40);
    this.nC.addChild(borderDebaff, 1460, 100, 40, 40);
    //-------------------------
    let arrow = new ImageButton().setImage("strelka.png");
    arrow.setImageSize(50);
    this.nC.addChild(arrow, 1525, 25, 50, 50);
    //-------------------------
    let t = this;
    arrow.onClicked.add(function () {
      t.HideUI();
      skillsMajor.ShowUI();
    });
  }

  HideUI() {
    this.nCUI.position = this.hidePosition;
    this.parent.updateUI(this.nCUI);
  }
  ShowUI() {
    this.nCUI.position = this.showPosition;
    this.parent.updateUI(this.nCUI);
  }
}
let skillsChanger = new SkillsChanger(refObject, new Vector(0, 0, 0));
//-----------------------------------------------------------------
function saveState() {
  let state = {};

  for (let i = 0; i < countSkills; i++) {
    state["major" + i] = majorSkills[index].getText();
    state["baff" + i] = baffSkills[index].getText();
    state["debaff" + i] = debaffSkills[index].getText();
    state["main" + i] = mainSkills[index].getText();
  }

  refObject.setSavedData(JSON.stringify(state));
}

function loadState() {
  if (refObject.getSavedData() === "") {
    saveState();
    return;
  }

  let state = JSON.parse(refObject.getSavedData());
  for (let i = 0; i < countSkills; i++) {
    majorSkills["major" + i].setText(state["major" + i] || "0");
    baffSkills["baff" + i].setText(state["baff" + i] || "0");
    debaffSkills["debaff" + i].setText(state["debaff" + i] || "0");
    mainSkills["main" + i].setText(state["main" + i] || "0");
  }
}

refObject.ResetValue = function () {
  for (let i = 0; i < countSkills; i++) {
    majorSkills["major" + i].setText("0");
    baffSkills["baff" + i].setText("0");
    debaffSkills["debaff" + i].setText("0");
    mainSkills["main" + i].setText("0");
    startValueMainSkills[i] = 0;
  }
}

refObject.RecalculationMain = (array) => {
  startValueMainSkills[0] = 5 + array["dextery"] * 4;
  startValueMainSkills[1] = array["dextery"] * 2;
  startValueMainSkills[2] = array["dextery"] * 2;
  startValueMainSkills[3] = 5 + array["dextery"] * 4;
  startValueMainSkills[4] = 30 + (array["dextery"] + array["strenght"]) * 2;
  startValueMainSkills[5] = 20 + (array["dextery"] + array["strenght"]) * 2;
  startValueMainSkills[6] = array["dextery"] * 4;
  startValueMainSkills[7] = (array["perception"] + array["intelligence"]) * 2;
  startValueMainSkills[8] = parseInt(((array["perception"] + array["intelligence"]) / 3)) * 5;
  startValueMainSkills[9] = (array["perception"] + array["intelligence"]) * 2;
  startValueMainSkills[10] = 5 + array["dextery"] * 3;
  startValueMainSkills[11] = 10 + array["dextery"] + array["perception"];
  startValueMainSkills[12] = array["dextery"] * 3;
  startValueMainSkills[13] = 10 + array["dextery"] + array["perception"];
  startValueMainSkills[14] = array["intelligence"] * 4;
  startValueMainSkills[15] = (array["perception"] + array["intelligence"]) * 3;
  startValueMainSkills[16] = array["charism"] * 5;
  startValueMainSkills[17] = array["charism"] * 4;
  startValueMainSkills[18] = array["luck"] * 5;
  for (let i = 0; i < countSkills; i++) {
    RecalculationMajorSkills(i);
  }
}
//-----------------------------------------------------------------
loadState();